@@ -1,18 +1,24 @@
 """ init for datacraft """
 
+import logging
+
 # model classes that may be implemented externally
 from .supplier.model import (
-    DataSpec, ValueSupplierInterface, Distribution, CasterInterface, RecordProcessor, OutputHandlerInterface)
+    DataSpec, ValueSupplierInterface, Distribution, CasterInterface, RecordProcessor, OutputHandlerInterface,
+    ResettableIterator)
 # programmatic spec building
 from . import builder
 # expose this at root too
-from .builder import spec_builder, parse_spec
+from .builder import spec_builder, parse_spec, entries
 # registry decorators
 from .registries import Registry as registry
 # exceptions and errors thrown
 from .exceptions import SpecException, ResourceError
 from .supplier.exceptions import SupplierException
 # commonly used by client code
-from .loader import Loader
+from .loader import field_loader, Loader
+from . import suppliers, distributions, outputs
 # to trigger registered functions
 from . import cli
+
+_log = logging.getLogger('datacraft.init')
