@@ -2,7 +2,7 @@ import re
 
 import pytest
 
-from datacraft import builder, Loader
+from datacraft import builder, field_loader
 # to trigger registration
 from datacraft import cli
 
@@ -12,7 +12,7 @@ UUID_REGEX = re.compile('[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f
 
 def test_uuid_spec():
     spec = builder.single_field("foo:uuid", {}).build()
-    loader = Loader(spec)
+    loader = field_loader(spec)
     supplier = loader.get('foo')
 
     value1 = supplier.next(0)
@@ -36,7 +36,7 @@ valid_data_specs = [
 def test_uuid_valid_schema(key, spec):
     # for coverage
     spec = builder.single_field(key, spec).build()
-    loader = Loader(spec, enforce_schema=True)
+    loader = field_loader(spec, enforce_schema=True)
     supplier = loader.get('foo')
 
     value1 = supplier.next(0)
